@@ -12,14 +12,14 @@
 ##############################################################################
 """ Information about customizable actions.
 
-$Id: ActionInformation.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: ActionInformation.py 110577 2010-04-07 06:33:17Z jens $
 """
 
 from UserDict import UserDict
 
-from AccessControl import ClassSecurityInfo
+from AccessControl.SecurityInfo import ClassSecurityInfo
 from Acquisition import aq_base, aq_inner, aq_parent
-from Globals import InitializeClass
+from App.class_init import InitializeClass
 from OFS.ObjectManager import IFAwareObjectManager
 from OFS.OrderedFolder import OrderedFolder
 from OFS.PropertyManager import PropertyManager
@@ -27,14 +27,13 @@ from OFS.SimpleItem import SimpleItem
 from zope.i18nmessageid import Message
 from zope.interface import implements
 
-from Expression import Expression
-from interfaces import IAction
-from interfaces import IActionCategory
-from interfaces import IActionInfo
-from interfaces.portal_actions import ActionInfo as z2IActionInfo
-from permissions import View
-from utils import _checkPermission
-from utils import getToolByName
+from Products.CMFCore.Expression import Expression
+from Products.CMFCore.interfaces import IAction
+from Products.CMFCore.interfaces import IActionCategory
+from Products.CMFCore.interfaces import IActionInfo
+from Products.CMFCore.permissions import View
+from Products.CMFCore.utils import _checkPermission
+from Products.CMFCore.utils import getToolByName
 
 _unchanged = [] # marker
 
@@ -45,7 +44,6 @@ class ActionCategory(IFAwareObjectManager, OrderedFolder):
     """
 
     implements(IActionCategory)
-    __implements__ = OrderedFolder.__implements__
 
     _product_interfaces = (IActionCategory, IAction)
 
@@ -76,6 +74,7 @@ class Action(PropertyManager, SimpleItem):
     implements(IAction)
 
     i18n_domain = 'cmf_default'
+    link_target = ''
 
     security = ClassSecurityInfo()
 
@@ -88,6 +87,8 @@ class Action(PropertyManager, SimpleItem):
          'label':'I18n Domain'},
         {'id': 'url_expr', 'type': 'string', 'mode': 'w',
          'label': 'URL (Expression)'},
+        {'id':'link_target', 'type': 'string', 'mode':'w',
+         'label':'Link Target'},
         {'id': 'icon_expr', 'type': 'string', 'mode': 'w',
          'label': 'Icon (Expression)'},
         {'id': 'available_expr', 'type': 'string', 'mode': 'w',
@@ -108,6 +109,7 @@ class Action(PropertyManager, SimpleItem):
         self._setPropValue( 'description', kw.get('description', '') )
         self._setPropValue( 'i18n_domain', kw.get('i18n_domain', '') )
         self._setPropValue( 'url_expr', kw.get('url_expr', '') )
+        self._setPropValue( 'link_target', kw.get('link_target', '') )
         self._setPropValue( 'icon_expr', kw.get('icon_expr', '') )
         self._setPropValue( 'available_expr', kw.get('available_expr', '') )
         self._setPropValue( 'permissions', kw.get('permissions', () ) )
@@ -118,8 +120,12 @@ class Action(PropertyManager, SimpleItem):
         if isinstance(value, list):
             value = tuple(value)
         setattr(self, id, value)
-        if value and id.endswith('_expr'):
-            setattr( self, '%s_object' % id, Expression(value) )
+        if id.endswith('_expr'):
+            attr = '%s_object' % id
+            if value:
+                setattr(self, attr, Expression(value))
+            elif hasattr(self, attr):
+                delattr(self, attr)
 
     security.declarePrivate('getInfoData')
     def getInfoData(self):
@@ -147,6 +153,8 @@ class Action(PropertyManager, SimpleItem):
                     val = True
             elif id == 'i18n_domain':
                 continue
+            elif id == 'link_target':
+                val = val or None
             elif self.i18n_domain and id in ('title', 'description'):
                 val = Message(val, self.i18n_domain)
             lazy_map[id] = val
@@ -162,7 +170,6 @@ class ActionInfo(UserDict):
     """
 
     implements(IActionInfo)
-    __implements__ = z2IActionInfo
 
     __allow_access_to_unprotected_subobjects__ = 1
 
@@ -176,6 +183,8 @@ class ActionInfo(UserDict):
                 self.data.setdefault( 'title', self.data['name'] )
                 del self.data['name']
             self.data.setdefault( 'url', '' )
+            self.data.setdefault( 'link_target', None )
+            self.data.setdefault( 'icon', '' )
             self.data.setdefault( 'category', 'object' )
             self.data.setdefault( 'visible', True )
             self.data['available'] = True
@@ -184,7 +193,7 @@ class ActionInfo(UserDict):
             (lazy_map, lazy_keys) = action.getInfoData()
             UserDict.__init__(self, lazy_map)
 
-        self.data['allowed'] = True
+        self.data.setdefault('allowed', True)
         permissions = self.data.pop( 'permissions', () )
         if permissions:
             self.data['allowed'] = self._checkPermissions
@@ -203,7 +212,7 @@ class ActionInfo(UserDict):
 
     def __eq__(self, other):
         # this is expensive, use it with care
-        [ self.__getitem__(key) for key in self._lazy_keys ]
+        [ self.__getitem__(key) for key in self._lazy_keys[:] ]
 
         if isinstance(other, self.__class__):
             [ other[key] for key in other._lazy_keys ]
@@ -250,6 +259,8 @@ class ActionInformation( SimpleItem ):
 
     implements(IAction)
 
+    link_target = ''
+
     __allow_access_to_unprotected_subobjects__ = 1
 
     security = ClassSecurityInfo()
@@ -264,6 +275,8 @@ class ActionInformation( SimpleItem ):
                 , priority=10
                 , visible=True
                 , action=''
+                , icon_expr=''
+                , link_target=''
                 ):
         """ Set up an instance.
         """
@@ -276,6 +289,8 @@ class ActionInformation( SimpleItem ):
                  , priority
                  , visible
                  , action
+                 , icon_expr
+                 , link_target
                  )
 
     security.declarePrivate('edit')
@@ -289,6 +304,8 @@ class ActionInformation( SimpleItem ):
             , priority=_unchanged
             , visible=_unchanged
             , action=_unchanged
+            , icon_expr=_unchanged
+            , link_target=_unchanged
             ):
         """Edit the specified properties.
         """
@@ -317,6 +334,12 @@ class ActionInformation( SimpleItem ):
             if action and isinstance(action, basestring):
                 action = Expression(action)
             self.setActionExpression(action)
+        if icon_expr is not _unchanged:
+            if icon_expr and isinstance(icon_expr, basestring):
+                icon_expr = Expression(icon_expr)
+            self.setIconExpression(icon_expr)
+        if link_target is not _unchanged:
+            self.link_target = link_target
 
     security.declareProtected( View, 'Title' )
     def Title(self):
@@ -388,6 +411,36 @@ class ActionInformation( SimpleItem ):
             action = Expression( action )
         self.action = action
 
+    security.declarePrivate( '_getIconExpressionObject' )
+    def _getIconExpressionObject( self ):
+
+        """ Find the icon expression object, working around name changes.
+        """
+        return getattr( self, 'icon_expr', None )
+
+    security.declarePublic( 'getIconExpression' )
+    def getIconExpression( self ):
+
+        """ Return the text of the TALES expression for our icon URL.
+        """
+        icon_expr = self._getIconExpressionObject()
+        expr = icon_expr and icon_expr.text or ''
+        if expr and isinstance(expr, basestring):
+            if ( not expr.startswith('string:')
+                 and not expr.startswith('python:') ):
+                expr = 'string:${object_url}/%s' % expr
+                self.icon_expr = Expression( expr )
+        return expr
+
+    security.declarePrivate( 'setIconExpression' )
+    def setIconExpression(self, icon_expr):
+        if icon_expr and isinstance(icon_expr, basestring):
+            if ( not icon_expr.startswith('string:')
+                 and not icon_expr.startswith('python:') ):
+                icon_expr = 'string:${object_url}/%s' % icon_expr
+            icon_expr = Expression( icon_expr )
+        self.icon_expr = icon_expr
+
     security.declarePublic( 'getCondition' )
     def getCondition(self):
 
@@ -418,6 +471,12 @@ class ActionInformation( SimpleItem ):
         """
         return bool(self.visible)
 
+    security.declarePublic('getLinkTarget')
+    def getLinkTarget(self):
+        """ Return the rendered link tag's target attribute value
+        """
+        return self.link_target
+
     security.declarePrivate('getMapping')
     def getMapping(self):
         """ Get a mapping of this object's data.
@@ -430,7 +489,9 @@ class ActionInformation( SimpleItem ):
                               and self.condition.text or '',
                  'permissions': self.permissions,
                  'visible': bool(self.visible),
-                 'action': self.getActionExpression() }
+                 'action': self.getActionExpression(),
+                 'icon_expr' : self.getIconExpression(),
+                 'link_target' : self.getLinkTarget() }
 
     security.declarePrivate('clone')
     def clone( self ):
@@ -445,6 +506,9 @@ class ActionInformation( SimpleItem ):
         lazy_keys = []
         lazy_map = self.getMapping()
 
+        if not lazy_map['link_target']:
+            lazy_map['link_target'] = None
+
         if lazy_map['action']:
             lazy_map['url'] = self._getActionObject()
             lazy_keys.append('url')
@@ -452,6 +516,13 @@ class ActionInformation( SimpleItem ):
             lazy_map['url'] = ''
         del lazy_map['action']
 
+        if lazy_map['icon_expr']:
+            lazy_map['icon'] = self._getIconExpressionObject()
+            lazy_keys.append('icon')
+        else:
+            lazy_map['icon'] = ''
+        del lazy_map['icon_expr']
+
         if lazy_map['condition']:
             lazy_map['available'] = self.testCondition
             lazy_keys.append('available')
