@@ -12,7 +12,7 @@
 ##############################################################################
 """ PortalFolder: CMF-enabled Folder objects.
 
-$Id: PortalFolder.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: PortalFolder.py 110577 2010-04-07 06:33:17Z jens $
 """
 
 import base64
@@ -20,43 +20,40 @@ import marshal
 import re
 from warnings import warn
 
-from AccessControl import ClassSecurityInfo
-from AccessControl import getSecurityManager
+from AccessControl.SecurityInfo import ClassSecurityInfo
+from AccessControl.SecurityManagement import getSecurityManager
 from Acquisition import aq_parent, aq_inner, aq_base
-from Globals import InitializeClass
+from App.class_init import InitializeClass
 from OFS.Folder import Folder
 from OFS.OrderSupport import OrderSupport
 from zope.component.factory import Factory
 from zope.interface import implements
 
-from CMFCatalogAware import CMFCatalogAware
-from DynamicType import DynamicType
-from exceptions import AccessControl_Unauthorized
-from exceptions import BadRequest
-from exceptions import zExceptions_Unauthorized
-from interfaces import IFolderish
-from interfaces import IMutableMinimalDublinCore
-from interfaces import ISiteRoot
-from interfaces.Folderish import Folderish as z2IFolderish
-from permissions import AddPortalContent
-from permissions import AddPortalFolders
-from permissions import DeleteObjects
-from permissions import ListFolderContents
-from permissions import ManagePortal
-from permissions import ManageProperties
-from permissions import View
-from utils import _checkPermission
-from utils import getToolByName
+from Products.CMFCore.CMFCatalogAware import OpaqueItemManager
+from Products.CMFCore.DynamicType import DynamicType
+from Products.CMFCore.exceptions import AccessControl_Unauthorized
+from Products.CMFCore.exceptions import BadRequest
+from Products.CMFCore.exceptions import zExceptions_Unauthorized
+from Products.CMFCore.interfaces import IFolderish
+from Products.CMFCore.interfaces import IMutableMinimalDublinCore
+from Products.CMFCore.interfaces import ISiteRoot
+from Products.CMFCore.permissions import AddPortalContent
+from Products.CMFCore.permissions import AddPortalFolders
+from Products.CMFCore.permissions import DeleteObjects
+from Products.CMFCore.permissions import ListFolderContents
+from Products.CMFCore.permissions import ManagePortal
+from Products.CMFCore.permissions import ManageProperties
+from Products.CMFCore.permissions import View
+from Products.CMFCore.utils import _checkPermission
+from Products.CMFCore.utils import getToolByName
 
 
-class PortalFolderBase(DynamicType, CMFCatalogAware, Folder):
+class PortalFolderBase(DynamicType, OpaqueItemManager, Folder):
 
     """Base class for portal folder.
     """
 
     implements(IFolderish, IMutableMinimalDublinCore)
-    __implements__ = (z2IFolderish, DynamicType.__implements__,
-                      Folder.__implements__)
 
     security = ClassSecurityInfo()
 
@@ -65,8 +62,9 @@ class PortalFolderBase(DynamicType, CMFCatalogAware, Folder):
     manage_options = ( Folder.manage_options[:1]
                      + ({'label': 'Components',
                          'action': 'manage_components'},)
+                     + ({'label': 'Components Folder',
+                         'action': '++etc++site/manage_main'},)
                      + Folder.manage_options[1:]
-                     + CMFCatalogAware.manage_options
                      )
 
     def __init__(self, id, title='', description=''):
@@ -118,7 +116,9 @@ class PortalFolderBase(DynamicType, CMFCatalogAware, Folder):
         """
         self.setTitle( title )
         self.setDescription( description )
-        self.reindexObject()
+        # BBB: for ICatalogAware subclasses
+        if getattr(self, 'reindexObject', None) is not None:
+            self.reindexObject()
 
     security.declarePublic('allowedContentTypes')
     def allowedContentTypes( self ):
@@ -258,21 +258,6 @@ class PortalFolderBase(DynamicType, CMFCatalogAware, Folder):
         """
         return None
 
-    # Ensure pure PortalFolders don't get cataloged.
-    # XXX We may want to revisit this.
-
-    def indexObject(self):
-        pass
-
-    def unindexObject(self):
-        pass
-
-    def reindexObject(self, idxs=[]):
-        pass
-
-    def reindexObjectSecurity(self):
-        pass
-
     def PUT_factory( self, name, typ, body ):
         """ Factory for PUT requests to objects which do not yet exist.
 
@@ -438,6 +423,12 @@ class PortalFolderBase(DynamicType, CMFCatalogAware, Folder):
                     raise ValueError('Disallowed subobject type: %s'
                                         % type_name)
 
+                # Check for workflow guards
+                objType = pt.getTypeInfo(type_name)
+                if ( objType is not None and
+                     not objType._checkWorkflowAllowed(self) ):
+                    raise ValueError('Pasting not allowed in this workflow')
+
     security.setPermissionDefault(AddPortalContent, ('Owner','Manager'))
 
     security.declareProtected(AddPortalFolders, 'manage_addFolder')
@@ -477,8 +468,6 @@ class PortalFolder(OrderSupport, PortalFolderBase):
     """Implements portal content management, but not UI details.
     """
 
-    __implements__ = (PortalFolderBase.__implements__,
-                      OrderSupport.__implements__)
     portal_type = 'Folder'
 
     security = ClassSecurityInfo()
@@ -491,7 +480,7 @@ class PortalFolder(OrderSupport, PortalFolderBase):
         """Add a new PortalFolder object with id *id*.
         """
         ob = PortalFolder(id, title)
-        self._setObject(id, ob)
+        self._setObject(id, ob, suppress_events=True)
         if REQUEST is not None:
             return self.folder_contents( # XXX: ick!
                 self, REQUEST, portal_status_message="Folder added")
